function Q=ardlpmg_full(y,x)
% y is T x n matrix of observations on the dependent variable
% x is T x n x k matrix of observations on regressors
% it is allowed for NaNs (at the begining or end)
% ARDL(1,1) models are assumed

[T,n]=size(y);
k=size(x,3);

% estimation
sx=zeros(k,k); sy=zeros(k,1);
Tb=0;
for i=1:n
 xi=squeeze(x(:,i,:));
 yi=y(:,i);
 % find available sample (time periods with NaNs are dropped)
 ni=isnan([yi,xi]);
 omit=(sum(ni,2)>0);
 yi=yi(not(omit));
 xi=xi(not(omit),:);
 Ti=size(yi,1);
 %ARDL-PMG matrices
 Wi=[yi(2:Ti,1)-yi(1:Ti-1,1),xi(2:Ti,:)-xi(1:Ti-1,:), ones(Ti-1,1)]; 
 Hi=[yi(1:Ti-1,1),xi(2:Ti,:),xi(1:Ti-1,:), ones(Ti-1,1)]; %, ones(T-1,1)
 P=Hi*inv(Hi'*Hi)*Hi';
 M=P-P*Wi*inv(Wi'*P*Wi)*Wi'*P;

sx=sx+xi(2:Ti,:)'*M*xi(2:Ti,:);
sy=sy+xi(2:Ti,:)'*M*yi(2:Ti,1);
Tb=Tb+Ti/n;
end
betahat=inv(sx)*sy;

% estimate coef and report residuals
    % ardl coefs
alpha=zeros(n,1); delta=zeros(n,k); c=zeros(n,1); 
    % coefs for the VAR model in dx
xphi=zeros(k,k,n); cx=zeros(k,n); u=NaN(T,k,n);

tau=[1:T]';
v=zeros(T,n);
sample=zeros(n,2); %Tstart, Tend
for i=1:n
    xi=squeeze(x(:,i,:));
    yi=y(:,i); 
    % find available sample (time periods with NaNs are dropped)
    ni=isnan([yi,xi]);
    omit=(sum(ni,2)>0);
    yi=yi(not(omit));
    xi=xi(not(omit),:);
    indi=tau(not(omit),1);
    Ti=size(yi,1);
    
    dyi=yi(2:Ti,1)-yi(1:Ti-1,1);
    ex=yi-xi*betahat;
    X=[ex(1:end-1,1),ones(Ti-1,1),xi(2:Ti,:)-xi(1:Ti-1,:)];
    b=X\dyi;
    alpha(i,1)=b(1,1);
    delta(i,:)=b(3:end,1);
    c(i,1)=b(2,1);
    v(indi(2:end),i)=(dyi-X*b)';
    sample(i,1)=min(indi);
    sample(i,2)=max(indi);
    % now marginal VAR for dx
    dxi=xi(2:Ti,:)-xi(1:Ti-1,:);
    for j=1:k
        dxr=[ones(Ti-2,1),dxi(1:Ti-2,:)];
       q=dxr\dxi(2:Ti-1,j);
       cx(j,i)=q(1,1);
       xphi(j,:,i)=q(2:end,1)';
       uij=dxi(2:Ti-1,j)-dxr*q;
       u(sample(i,1)+2:sample(i,2),j,i)=uij;
    end
end
%u=x(2:T,:)-x(1:T-1,:);

% assign values
Q.beta=betahat;
Q.alpha=alpha;
Q.delta=delta;
Q.c=c;
Q.v=v;
Q.u=u;
Q.xphi=xphi;
Q.cx=cx;
Q.sx=sx;
Q.sample=sample;
return